@@ -40,6 +40,7 @@
 #include "gp_time.h"
 #include "term_api.h"
 #include "gplocale.h"
+#include "mouse.h"	/* for inside_zoom() */
 
 /* HBB 20000725: gather all per-axis variables into a struct, and set
  * up a single large array of such structs. Next step might be to use
@@ -167,11 +168,6 @@ int widest_tic_strlen;
 /* Number of axis tics actually placed, set by tic_count_callback() */
 static int axis_tic_count;
 
-/* flag to indicate that in-line axis ranges should be ignored
- * and zoom/pan range limits take precedence over auto-scaling
- */
-TBOOLEAN inside_zoom;
-
 /* axes being used by the current plot */
 /* These are mainly convenience variables, replacing separate copies of
  * such variables originally found in the 2D and 3D plotting code */
@@ -429,7 +425,7 @@ axis_checked_extend_empty_range(AXIS_INDEX axis, const char *mesg)
 
     if (dmax - dmin == 0.0) {
 	/* empty range */
-	if (this_axis->autoscale || inside_zoom) {
+	if (this_axis->autoscale || inside_zoom()) {
 	    /* range came from autoscaling ==> widen it */
 	    double widen = (dmax == 0.0) ?
 		FIXUP_RANGE__WIDEN_ZERO_ABS
@@ -438,9 +434,9 @@ axis_checked_extend_empty_range(AXIS_INDEX axis, const char *mesg)
 		fprintf(stderr, "Warning: empty %s range [%g:%g], ",
 		    axis_name(axis), dmin, dmax);
 	    /* HBB 20010525: correctly handle single-ended autoscaling */
-	    if ((this_axis->autoscale & AUTOSCALE_MIN) || inside_zoom)
+	    if ((this_axis->autoscale & AUTOSCALE_MIN) || inside_zoom())
 		this_axis->min -= widen;
-	    if ((this_axis->autoscale & AUTOSCALE_MAX) || inside_zoom)
+	    if ((this_axis->autoscale & AUTOSCALE_MAX) || inside_zoom())
 		this_axis->max += widen;
 	    if (!(axis == FIRST_Z_AXIS && !mesg)) /* set view map */
 		fprintf(stderr, "adjusting to [%g:%g]\n",
@@ -951,7 +947,7 @@ setup_tics(struct axis *this, int max)
     }
 
     /* It is disconcerting when the response to pan or zoom is asymmetric */
-    if (inside_zoom)
+    if (inside_zoom())
 	autoextend_min = autoextend_max = FALSE;
 
     /* If an explicit stepsize was set, axis->timelevel wasn't defined,
@@ -1448,6 +1444,16 @@ gen_tics(struct axis *this, tic_callback callback)
 		    }
 		}
 
+		/* If the axis is logscaled but the major tics have been changed
+		 * from log to linear placement, adjust the minor tics to match.
+		 */
+		if (nonlinear(this) && this->log
+		&&  (!def->logscaling || (def->logscaling && def->force_linear_tics))) {
+		    ministep = step / 10.;
+		    ministart = internal + ministep;
+		    miniend = internal + step;
+		}
+
 		for (mplace = ministart; mplace < miniend; mplace += ministep) {
 		    if (this->tictype == DT_TIMEDATE) {
 			mtic_user = time_tic_just(this->timelevel - 1, internal + mplace);
@@ -1464,7 +1470,12 @@ gen_tics(struct axis *this, tic_callback callback)
 			    mtic_user = this_major + mplace/miniend * (next_major - this_major);
 			    mtic_internal = eval_link_function(this->linked_to_primary, mtic_user);
 			}
+		    } else if (nonlinear(this) && this->log
+			   &&  (!def->logscaling || (def->logscaling && def->force_linear_tics))) {
+			mtic_user = mplace;
+			mtic_internal = mtic_user;	/* It isn't really but this makes the range checks work */
 		    } else if (nonlinear(this) && this->log) {
+			/* FIXME - not sure this is correct. Fall through instead? */
 			mtic_user = internal + mplace;
 			mtic_internal = eval_link_function(this->linked_to_primary, mtic_user);
 		    } else {
@@ -2061,6 +2072,7 @@ sanity_check_log_tics( int axis_index )
 	gen_tics(axis, tic_count_callback);	/* find actual number of tics */
 	if (axis_tic_count < 3) {
 	    axis->ticdef.force_linear_tics = TRUE;
+	    FPRINTF((stderr, " >>>>>>>> forcing switch to linear tic generation along %s\n", axis_name(axis_index)));
 	    setup_tics(axis, 20);
 	}
     }
@@ -2524,7 +2536,14 @@ clone_linked_axes(AXIS *axis1, AXIS *axis2)
     double testmin, testmax, scale;
     TBOOLEAN suspect = FALSE;
 
+    double save_data_min = axis2->data_min;
+    double save_data_max = axis2->data_max;
+
     memcpy(axis2, axis1, AXIS_CLONE_SIZE);
+
+    axis2->data_min = save_data_min;
+    axis2->data_max = save_data_max;
+
     if (axis2->link_udf == NULL || axis2->link_udf->at == NULL)
 	return;
 
@@ -2720,8 +2739,6 @@ update_primary_axis_range(struct axis *secondary)
 	/* nonlinear axis (secondary is visible; primary is hidden) */
 	primary->min = eval_link_function(primary, secondary->min);
 	primary->max = eval_link_function(primary, secondary->max);
-	primary->data_min = eval_link_function(primary, secondary->data_min);
-	primary->data_max = eval_link_function(primary, secondary->data_max);
     }
 }
 
@@ -2737,8 +2754,6 @@ update_secondary_axis_range(struct axis *primary)
     if (secondary != NULL) {
        secondary->min = eval_link_function(secondary, primary->min);
        secondary->max = eval_link_function(secondary, primary->max);
-       secondary->data_min = eval_link_function(secondary, primary->data_min);
-       secondary->data_max = eval_link_function(secondary, primary->data_max);
     }
 }
 
@@ -2757,32 +2772,30 @@ extend_autoscaled_log_axis(AXIS *primary)
 }
 
 /*
- * We track the x1 and x2 axis data limits separately.
- * However if x1 and x2 are linked to each other we must reconcile
- * their data limits before plotting.
+ * We track the x1/x2 and y1/y2 axis data limits separately.
+ * However if either pair is linked ("set link x2") both data ranges
+ * contribute to autoscaling the full plot.
  */
 void
 reconcile_linked_axes(AXIS *primary, AXIS *secondary)
 {
-    double dummy;
-    coord_type inrange = INRANGE;
     if ((primary->autoscale & AUTOSCALE_BOTH) != AUTOSCALE_NONE
     &&  primary->linked_to_secondary) {
-	double min_2_into_1 = eval_link_function(primary, secondary->data_min);
-	double max_2_into_1 = eval_link_function(primary, secondary->data_max);
-
-	/* Merge secondary min/max into primary data range */
-	store_and_update_range(&dummy, min_2_into_1, &inrange, primary, FALSE);
-	store_and_update_range(&dummy, max_2_into_1, &inrange, primary, FALSE);
-	(void)dummy;	/* Otherwise the compiler complains about an unused variable */
-
-	/* Take the result back the other way to update secondary */
-	secondary->min = eval_link_function(secondary, primary->min);
-	secondary->max = eval_link_function(secondary, primary->max);
+	double min_2 = eval_link_function(primary, secondary->data_min);
+	double max_2 = eval_link_function(primary, secondary->data_max);
+	double min_1 = eval_link_function(secondary, primary->data_min);
+	double max_1 = eval_link_function(secondary, primary->data_max);
+	if (primary->min > min_2)
+	    primary->min = min_2;
+	if (primary->max < max_2)
+	    primary->max = max_2;
+	if (secondary->min > min_1)
+	    secondary->min = min_1;
+	if (secondary->max < max_1)
+	    secondary->max = max_1;
     }
 }
 
-
 /*
  * Check for linked-axis coordinate transformation given by command
  *     set {x|y}2r link via <expr1> inverse <expr2>
